module Css exposing
  ( style, stylesheet
  , rule, declaration
  , Node, Rule, Selector, Declaration, Property, Value
  )

{-| A library to create and use css stylesheets in Elm.

# Type Definitions
@docs Node, Rule, Selector, Declaration, Property, Value

# Constructors
@docs stylesheet, rule, declaration

# Html Elements
@docs style
-}

import Html exposing (Html)
import String


{-------------------------------------------------------------------------------
                               Type Definitions
-------------------------------------------------------------------------------}


{-| A top level member of Css stylesheet.
-}
type alias Node = String


{-| A member of Css stylesheet. It has a selector and declarations (body).
-}
type alias Rule = String


{-| A member of Rule. Css Selector.
-}
type alias Selector = String


{-| A member of Rule. It has a property and it's value.
-}
type alias Declaration = String


{-| A member of Declaration. Css Property.
-}
type alias Property = String


{-| A member of Declaration. Css Value.
-}
type alias Value = String


{-------------------------------------------------------------------------------
                                  Constructors
-------------------------------------------------------------------------------}


{-| Constructs a Rule from a Selector and a List of Declaration.
-}
rule : Selector -> List Declaration -> Rule
rule selector declarations =
  selector ++ "{" ++ String.concat declarations ++ "}"


{-| Constructs a Declaration from a Property and a Value.
-}
declaration : Property -> Value -> Declaration
declaration property value =
  property ++ ":" ++ value ++ ";"


{-| Constructs a Html Style Element from a List of Node.
-}
stylesheet : List Node -> Html msg
stylesheet nodes =
  style [] (List.map (Html.text) nodes)


{-------------------------------------------------------------------------------
                                 HTML Elements
-------------------------------------------------------------------------------}


{-| Html style tag.
-}
style : List (Html.Attribute msg) -> List (Html msg) -> Html msg
style =
  Html.node "style"